// --------------------------------------------------------------------------
#ifndef TDirectDrawH
#define TDirectDrawH

// ==========================================================================
// File:            TDirectDraw.H
// Description:     This file contains the definition for the TDirectDraw
//                  component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the BCB related definitions, etc.
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
#include <vcl\System.hpp>

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"
#include "TDirectDrawMisc.h"

// --------------------------------------------------------------------------
// include the TDirectDraw-specific types
// --------------------------------------------------------------------------
#include "TDDDisplayMode.h"

// --------------------------------------------------------------------------
// include the various DirectX Wrapper classes used by TDirectDraw
// --------------------------------------------------------------------------
#include "TDDCaps.h"

// --------------------------------------------------------------------------
// include the various DirectX Component classes used by TDirectDraw
// --------------------------------------------------------------------------
#include "TDirectDrawClipper.h"
#include "TDirectDrawPalette.h"
#include "TDirectDrawSurface.h"
#include "TDirectDrawVideoPort.h"
#include "TDirectDrawVideoPortContainer.h"

// --------------------------------------------------------------------------
// Event Method:    TDirectDrawEnumCallback
// Description:     This is a single-purpose event method that is used with
//                  the TDirectDraw OnEnumCallback event
// --------------------------------------------------------------------------
typedef __fastcall  void (__closure* TDirectDrawEnumCallback)
                         ( TObject*     Sender,
                           GUID FAR*    TheGuid,
                           AnsiString   TheDriverDescription,
                           AnsiString   TheDriverName,
                           void*        TheContext );

// --------------------------------------------------------------------------
// Event Method:    TDirectDrawEnumCallbackEx
// Description:     This is a single-purpose event method that is used with
//                  the TDirectDraw OnEnumCallbackEx event
// --------------------------------------------------------------------------
typedef __fastcall  void (__closure* TDirectDrawEnumCallbackEx)
                         ( TObject*     Sender,
                           GUID FAR*    TheGuid,
                           AnsiString   TheDriverDescription,
                           AnsiString   TheDriverName,
                           void*        TheContext,
                           HMONITOR     TheMonitor );

// --------------------------------------------------------------------------
// Event Method:    TDirectDrawEnumModesCallback
// Description:     This is a single-purpose event method that is used with
//                  the TDirectDraw OnEnumModesCallback event
// --------------------------------------------------------------------------
typedef __fastcall  void (__closure* TDirectDrawEnumModesCallback)
                         ( TObject*             Sender,
                           TDDSurfaceDesc*      TheDescription,
                           void*                TheContext );

// --------------------------------------------------------------------------
// Event Method:    TDirectDrawEnumSurfacesCallback
// Description:     This is a single-purpose event method that is used with
//                  the TDirectDraw OnEnumSurfacesCallback event method
// --------------------------------------------------------------------------
typedef __fastcall  void (__closure* TDirectDrawEnumSurfacesCallback)
                         ( TObject*             Sender,
                           TDirectDrawSurface*  TheSurface,
                           TDDSurfaceDesc*      TheDescription,
                           void*                TheContext );

// --------------------------------------------------------------------------
// Event Method:    TDirectDrawEnumVideoCallback
// Description:     This is a single-purpose event method that is used with
//                  the TDirectDraw OnEnumVideoCallback event method
// --------------------------------------------------------------------------
typedef __fastcall  void (__closure* TDirectDrawEnumVideoCallback)
                         ( TObject*             Sender,
                           TDDVideoPortCaps*    TheVideoPortCaps,
                           void*                TheContext );

// ==========================================================================
//
// Class:           TDirectDraw
//
// Description:     This is a component that wraps the functionality of the
//                  IDIRECTDRAW2 interface available within DirectX5
//
//                  All methods of this object attempt to call the relevant
//                  IDIRECTDRAW2 interface functions. When a call to the
//                  IDIRECTDRAW2 function fails, some functions automatically
//                  attempt to call the IDIRECTDRAW version of the same
//                  function
//
//                  All methods call the OnError event method if any errors
//                  are encountered during execution of the relevant DirectX
//                  functions
//
// Published        BackBuffer
// Properties:      BitsPerPixel
//                  CooperativeLevel
//                  DisplayMode
//                  Height
//                  PrimarySurface
//                  RefreshRate
//                  Width
//
// Public           Created
// Properties:      CurrentDisplayModeInfo
//                  DriverCaps
//                  // GDISurface
//                  HelCaps
//                  MonitorFrequency
//                  Ptr
//                  ScanLine
//                  UseFormHandle
//                  VerticalBlankStatus
//
// Methods:         Create()
//                  CreateClipper()
//                  CreatePalette()
//                  CreateSurface()
//                  DirectDrawCreate()
//                  DirectDrawCreateClipper()
//                  DuplicateSurface()
//                  EnumDisplayModes()
//                  EnumSurfaces()
//                  ErrorHandler()
//                  FlipToGDISurface()
//                  GetAvailableVidMem()
//                  GetCaps()
//                  GetDisplayMode()
//                  GetFourCCCodes()
//                  GetGDISurface()
//                  GetMonitorFrequency()
//                  GetScanLine()
//                  GetVerticalBlankStatus()
//                  MessageHandler()
//                  Release()
//                  RestoreDisplayMode()
//                  SetCooperativeLevel()
//                  SetDisplayMode()
//                  WaitForVerticalBlank()
//
// Events:          OnCreate()
//                  OnEnumCallback()
//                  OnEnumCallbackEx()
//                  OnEnumModesCallback()
//                  OnEnumSurfacesCallback()
//                  OnEnumVideoPortCallback()
//                  OnError()
//                  OnMessage()
//                  OnRelease()
//
// ==========================================================================
class TDirectDraw : public TComponent
{
__published:

// --------------------------------------------------------------------------
// Property:        BackBuffer
// Description:     If this property has been assigned before the Create()
//                  method is called and the PrimarySurface property has also
//                  been assigned, the internals of the supplied
//                  TDirectDrawSurface component are automatically created
//                  during the call to Create()
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface*  BackBuffer = { read=FGetBackBuffer,
                                                        write=FSetBackBuffer,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        BitsPerPixel
// Description:     This is the default number of bits used per pixel
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
    __property      dword              BitsPerPixel = {
                                                    read=FGetBitsPerPixel,
                                                    write=FSetBitsPerPixel,
                                                    default=16 };

// --------------------------------------------------------------------------
// Property:        CooperativeLevel
// Description:     Used to set the current CooperativeLevel of the
//                  internal IDIRECTDRAW2 interface
// Note:            Setting this property is equivalent to calling the
//                  SetCooperativeLevel() method with the default window
//                  handle as defined with the UseFormHandle property
// --------------------------------------------------------------------------
    __property      dword          CooperativeLevel = {
                                                read=FGetCooperativeLevel,
                                                write=FSetCooperativeLevel,
                                                nodefault };

// --------------------------------------------------------------------------
// Property:        DisplayMode
// Description:     Used to easily set the current display mode to one of a
//                  list of standard display modes
// Note:            Examples _640x480x8, _800x600x16, etc
// --------------------------------------------------------------------------
    __property      TDDDisplayMode      DisplayMode = {
                                                    read=FGetDisplayMode,
                                                    write=FSetDisplayMode
                                                    ,default=_800x600x16 };

// --------------------------------------------------------------------------
// Property:        Height
// Description:     This is the default height of the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
    __property      dword                    Height = { read=FGetHeight,
                                                        write=FSetHeight,
                                                        default=600 };

// --------------------------------------------------------------------------
// Event:           OnCreate
// Description:     This event method is called when the Create() method has
//                  created the IDIRECTDRAW and IDIRECTDRAW2 interfaces
//                  successfully
// Note:            This is handy for checking capabilities, creating
//                  surfaces, etc
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent       OnCreate = { read=FOnCreate,
                                                        write=FOnCreate,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnEnumCallback
// Description:     This event method is called during the execution of the
//                  DirectDrawEnumerate() method
// --------------------------------------------------------------------------
    __property  TDirectDrawEnumCallback
                                     OnEnumCallback = { read=FOnEnumCallback,
                                                        write=FOnEnumCallback
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Event:           OnEnumCallbackEx
// Description:     This event method is called during the execution of the
//                  DirectDrawEnumerateEx() method
// --------------------------------------------------------------------------
    __property  TDirectDrawEnumCallbackEx
                                   OnEnumCallbackEx = {
                                                    read=FOnEnumCallbackEx,
                                                    write=FOnEnumCallbackEx,
                                                    nodefault };

// --------------------------------------------------------------------------
// Event:           OnEnumModesCallback
// Description:     This event method is called during the execution of the
//                  EnumDisplayModes() method
// --------------------------------------------------------------------------
    __property  TDirectDrawEnumModesCallback
                                OnEnumModesCallback = {
                                                read=FOnEnumModesCallback,
                                                write=FOnEnumModesCallback,
                                                nodefault };

// --------------------------------------------------------------------------
// Event:           OnEnumSurfacesCallback
// Description:     This event method is called during the execution of the
//                  EnumSurfaces() method
// --------------------------------------------------------------------------
    __property  TDirectDrawEnumSurfacesCallback
                             OnEnumSurfacesCallback = {
                                                read=FOnEnumSurfacesCallback,
                                                write=FOnEnumSurfacesCallback
                                                ,nodefault };

// --------------------------------------------------------------------------
// Event:           OnEnumVideoCallback
// Description:     This event method is called during the execution of the
//                  ???EnumVideoPorts() method
// --------------------------------------------------------------------------
    __property  TDirectDrawEnumVideoCallback
                                OnEnumVideoCallback = {
                                                read=FOnEnumVideoCallback,
                                                write=FOnEnumVideoCallback,
                                                nodefault };

// --------------------------------------------------------------------------
// Event:           OnError
// Description:     This event method is called when a valid DirectX error
//                  occurs
// Note:            TDirectDraw supplies a default OnError event handler
//                  which opens a simple message box to display the details
//                  of the error. To stop TDirectDraw Error messages from
//                  occurring, set this property to NULL after you call
//                  Create()
// --------------------------------------------------------------------------
    __property	    TDirectDrawError	    OnError = { read=FOnError,
                                                        write=FOnError,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnMessage
// Description:     This event method is called when the TDirectDraw object
//                  performs any kind of task
// Note:            This is incredibly useful for debugging.
//                  TDirectDraw supplies a default OnMessage event handler
//                  which opens a simple message box to display the details
//                  of the error. To stop TDirectDraw messages from occurring
//                  set this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property      TDirectDrawMessage    OnMessage = { read=FOnMessage,
                                                        write=FOnMessage,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnRelease
// Description:     This event method is called before the Release() method
//                  releases the IDIRECTDRAW and IDIRECTDRAW2 interfaces
// Note:            This is handy for releasing surfaces, etc
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent      OnRelease = { read=FOnRelease,
                                                        write=FOnRelease,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        PrimarySurface
// Description:     If this property has been assigned before the Create()
//                  method is called, the internals of the supplied
//                  TDirectDrawSurface component are automatically created
//                  during the call to Create()
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface*
                                     PrimarySurface = {
                                                    read=FGetPrimarySurface,
                                                    write=FSetPrimarySurface,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        RefreshRate
// Description:     This is the default refresh rate for the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
    __property      dword               RefreshRate = { read=FGetRefreshRate,
                                                        write=FSetRefreshRate
                                                        ,default=60 };

// --------------------------------------------------------------------------
// Property:        Width
// Description:     This is the default width of the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
    __property      dword                     Width = { read=FGetWidth,
                                                        write=FSetWidth,
                                                        default=800 };

public:

// --------------------------------------------------------------------------
// Property:        Created
// Description:     This property is true if the Create() function has been
//                  called and has successfully created the IDIRECTDRAW and
//                  IDIRECTDRAW2 interface
// --------------------------------------------------------------------------
    __property      bool                    Created = { read=fCreated };

// --------------------------------------------------------------------------
// Property:        CurrentDisplayModeInfo
// Description:     Returns a newly created TDDSurfaceDesc object containing
//                  the current display mode information
// Notes:           Returns NULL if the directx call failed
//                  Remember to 'delete' the newly created TDDSurfaceDesc*
// --------------------------------------------------------------------------
    __property      TDDSurfaceDesc*
                             CurrentDisplayModeInfo = {
                                        read=FGetCurrentDisplayModeInfo,
                                        write=FSetCurrentDisplayModeInfo,       //added write, nodefault
                                        nodefault };

// --------------------------------------------------------------------------
// Property:        DriverCaps
// Description:     Use this property to access the current capabilities of
//                  the display drivers
// --------------------------------------------------------------------------
    __property      TDDCaps*             DriverCaps = { read=fDriverCaps,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        GDISurface
// Description:     Returns a pointer to an existing TDirectDrawSurface*
//                  object which is the current GDI surface
// Note:            Returns NULL if the directx call failed
// --------------------------------------------------------------------------
//  __property    TDirectDrawSurface*    GDISurface = {
//                                                  read=FGetGDISurface };

// --------------------------------------------------------------------------
// Property:        HelCaps
// Description:     Use this property to access the current capabilities of
//                  the hardware emulation layer
// --------------------------------------------------------------------------
    __property      TDDCaps*                HelCaps = { read=fHelCaps,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        MonitorFrequency
// Description:     Returns the result of the GetMonitorFrequency() method
// Note:            Returns 0 if the directx call failed
// --------------------------------------------------------------------------
    __property      dword          MonitorFrequency = {
                                                read=FGetMonitorFrequency,
                                                write=FSetMonitorFrequency,     //added write, nodefault
                                                nodefault };

// --------------------------------------------------------------------------
// Property:        Ptr
// Description:     Use this property to access the directx LPDIRECTDRAW2
//                  variable
// --------------------------------------------------------------------------
    __property      LPDIRECTDRAW2               Ptr = { read=FGetPtr,
                                                        write=FSetPtr,
                                                        default=NULL };

// --------------------------------------------------------------------------
// Property:        ScanLine
// Description:     Returns the result of the GetScanLine() method
// Note:            Returns 0 if the directx call failed
// --------------------------------------------------------------------------
    __property      dword                  ScanLine = { read=FGetScanLine,
                                                        write=FSetScanLine,     //added write, nodefault
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        UseFormHandle
// Description:     This value modifies the default window handle used when
//                  setting the CooperativeLevel property
// Note:            If true, this TDirectDraw object is "owned" by the form,
//                  meaning that all the default CooperativeLevel values are
//                  set using the parent Form's ->Handle
//                  If false, this TDirectDraw object is "owned" by the
//                  application, meaning that the default CooperativeLevel
//                  values are set using the Application's ->Handle
// --------------------------------------------------------------------------
    __property      bool              UseFormHandle = {
                                                    read=FGetUseFormHandle,
                                                    write=FSetUseFormHandle,
                                                    default=true };

// --------------------------------------------------------------------------
// Property:        VerticalBlankStatus
// Description:     Returns the result of the GetVerticalBlankStatus() method
// Note:            Returns 0 if the directx call failed
// --------------------------------------------------------------------------
    __property      bool        VerticalBlankStatus = {
                                            read=FGetVerticalBlankStatus,
                                            write=FSetVerticalBlankStatus,      //added write, nodefault
                                            nodefault };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
					__fastcall	TDirectDraw(TComponent* Owner);
    virtual			__fastcall	~TDirectDraw();

// --------------------------------------------------------------------------
// Method:          Create
// Description:     Call this method to create the internals of the
//                  TDirectDraw object
// Notes:           Remember to call the Release() method when finished
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  Create();

// --------------------------------------------------------------------------
// Method:          CreateClipper
// Description:     Creates a DirectDrawClipper object
// Parameters:      Clipper - address of a pointer for storing the location
//                          of the new clipper
// --------------------------------------------------------------------------
	virtual	HRESULT __fastcall	CreateClipper( TDirectDrawClipper*
                                                                pClipper );
    // DWORD dwFlags, LPDIRECTDRAWCLIPPER FAR *lplpDDClipper, IUnknown FAR
    //                                                          *pUnkOuter

// --------------------------------------------------------------------------
// Method:          CreatePalette
// Description:     Creates a DirectDrawPalette for this DirectDraw object
// Parameters:      Palette - address of a pointer for the address of the new
//                          DirectDrawPalette object if successfully created
// --------------------------------------------------------------------------
    virtual	HRESULT __fastcall	CreatePalette( TDirectDrawPalette*
                                                                pPalette );
    // DWORD dwFlags, LPPALETTEENTRY lpColorTable, LPDIRECTDRAWPALETTE FAR
    //                              *lplpDDPalette, IUnknown FAR *pUnkOuter

// --------------------------------------------------------------------------
// Method:          CreateSurface
// Description:     Call this function to create a direct draw surface
// Parameters:      surface - an existing TDirectDrawSurface that is to be
//                  created
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	CreateSurface( TDirectDrawSurface*
                                                                pSurface );
    // LPDDSURFACEDESC lpDDSurfaceDesc, LPDIRECTDRAWSURFACE FAR
    //                              *lplpDDSurface, IUnknown FAR *pUnkOuter

// --------------------------------------------------------------------------
// Method:          DirectDrawCreate                                            //doesnt exist in cpp file
// Description:     This function can be used to manually create a DirectX
//                  interface
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  DirectDrawCreate() {}
    // GUID FAR *lpGUID, LPDIRECTDRAW FAR *lplpDD, IUnknown FAR *pUnkOuter

// --------------------------------------------------------------------------
// Method:          DirectDrawCreateClipper                                     //doesnt exist in cpp file
// Description:     Creates an independant DirectDrawClipper object
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  DirectDrawCreateClipper() {}
    // DWORD dwFlags, LPDIRECTDRAWCLIPPER FAR *lplpDDClipper, IUnknown FAR
    //                                                          *pUnkOuter

// --------------------------------------------------------------------------
// Method:          DirectDrawEnumerate                                         //doesnt exist in cpp file
// Description:     Enumerates the primary DirectDraw display device and a
//                  non-display device if one is installed.
// Note:            The NULL entry identifies the primary display device
//                  shared with GDI
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  DirectDrawEnumerate() {}
    // LPDDENUMCALLBACK lpCallback, LPVOID lpContext

// --------------------------------------------------------------------------
// Method:          DirectDrawEnumerateEx                                       //doesnt exist in cpp file
// Description:     Enumerates all DirectDraw devices installed on the system
// Note:            The NULL entry identifies the primary display device
//                  shared with GDI
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  DirectDrawEnumerateEx() {}
    // LPDDENUMCALLBACKEX lpCallback, LPVOID lpContext, DWORD  dwFlags

// --------------------------------------------------------------------------
// Method:          DuplicateSurface
// Description:     Duplicates a DirectDrawSurface object
// Parameters:      SourceSurface   - address of structure to be duplicated
//                  DuplicateSurface- address of pointer to the newly created
//                                  duplicate structure
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	DuplicateSurface(
                                    TDirectDrawSurface* pSourceSurface,
                                    TDirectDrawSurface* pDuplicateSurface );
    // LPDIRECTDRAWSURFACE lpDDSurface, LPLPDIRECTDRAWSURFACE FAR
    //                                                      *lplpDupDDSurface

// --------------------------------------------------------------------------
// Method:          EnumDisplayModes
// Description:     Call this function to "enumerate" the current display
//                  modes
// Note:            TDirectDraw will call the OnEnumDisplayModes event method
//                  for each of the currently available display modes
// Parameters:      flags   - can be used to modify the type of display mode
//                          enumerated
//                  desc    -  the options of this object can be used to
//                          determine the kind of display mode to be
//                          enumerated
//                  context - this is pointer to a user-defined object that
//                          will be passed directly to the OnEnumDisplayModes
//                          event method for each of the enumerated display
//                          modes
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	EnumDisplayModes( dword pFlags,
                                                TDDSurfaceDesc* pSurfaceDesc,
                                                void* pContext );
    // DWORD dwFlags, LPDDSURFACEDESC lpDDSurfaceDesc, LPVOID lpContext,
    //                              LPDDENUMMODESCALLBACK lpEnumModesCallback

// --------------------------------------------------------------------------
// Method:          EnumSurfaces
// Description:     Enumerates all of the existing or possible surfaces that
//                  meet the search criterion specified
// Parameters:      Flags       - type of enum search to be performed
//                  SurfaceDesc - address of a TDDSURFACEDESC structure that
//                              defines the surface of interest
//                  Context     - address of a structure that will be passed
//                              to each enumeration member
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	EnumSurfaces( dword pFlags,
                                                TDDSurfaceDesc* pSurfaceDesc,
                                                void* pContext );
    // DWORD dwFlags, LPDDSURFACEDESC lpDDSD, LPVOID lpContext,
    //                      LPDDENUMSURFACESCALLBACK lpEnumSurfacesCallback

// --------------------------------------------------------------------------
// Method:          ErrorHandler
// Description:     This is the TDirectDraw component's default error
//                  handling method which displays a simple dialog box with
//                  appropriate formatting of the error message
// Notes:           Use the OnError event to override default error handling
// --------------------------------------------------------------------------
    void            __fastcall  ErrorHandler( TObject *Sender,
                                                AnsiString TheFunction,
                                                AnsiString TheError,
                                                AnsiString TheErrorMessage );

// --------------------------------------------------------------------------
// Method:          FlipToGDISurface
// Description:     Flips the currently displayed surface so that the surface
//                  used by the Windows GDI is displayed
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	FlipToGDISurface();
    // no params

// --------------------------------------------------------------------------
// Method:          GetAvailableVidMem
// Description:     Call this function to retrieve the total memory and free
//                  memory available for a particular type of surface
// Parameters:      scaps   - an existing TDDSCaps object describing the type
//                          of surface that we want to get information about
//                  total   - a pointer to a local variable that is to be
//                          filled with total available memory (in bytes)
//                  free    - a pointer to a local variable that is to be
//                          filled with free available memory  (in bytes)
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall  GetAvailableVidMem( TDDSCaps* pSCaps,
                                                            dword* pTotal,
                                                            dword* pFree );
    // LPDDSCAPS lpDDSCaps, LPDWORD lpdwTotal, LPDWORD lpdwFree

// --------------------------------------------------------------------------
// Method:          GetCaps
// Description:     Call this function to retrieve the current direct draw
//                  capabilities
// Parameters:      drivercaps  - an existing TDDCaps object that is to be
//                              filled with the current driver capabilities
//                  helcaps     - an existing TDDCaps object that is to be
//                              filled with the current hardware emulation
//                              layer capabilities
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	GetCaps( TDDCaps* pDriverCaps,
                                                        TDDCaps* pHelCaps );
    // LPDDCAPS lpDDDriverCaps, LPDDCAPS lpDDHELCaps

// --------------------------------------------------------------------------
// Method:          GetDisplayMode
// Description:     Call this function to retrieve the current display mode
//                  description
// Parameters:      desc    - an existing TDDSurfaceDesc object that is to be
//                          filled with the current display mode description
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	GetDisplayMode( TDDSurfaceDesc*
                                                            pSurfaceDesc );
    // LPDDSURFACEDESC lpDDSurfaceDesc

// --------------------------------------------------------------------------
// Method:          GetFourCCCodes
// Description:     Retrieves FOURCC codes supported by the DirectDraw object
// Parameters:      NumCodes    - address with the number of codes supported
//                  Codes       - address of array for holding FOURCC codes
// Note:            This method can also retrieve the number of codes
//                  supported by setting Codes to NULL
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	GetFourCCCodes( dword* pNumCodes,
                                                            dword* pCodes );
    // LPDWORD lpNumCodes, LPDWORD lpCodes

// --------------------------------------------------------------------------
// Method:          GetGDISurface
// Description:     Retrieves the surface memory that GDI is treating as the
//                  primary surface
// Parameters:      GDISurface  - Address of a pointer to the
//                              DirectDrawSurface object that currently
//                              controls GDIs primary surface memory
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	GetGDISurface( TDirectDrawSurface*
                                                            pGDISurface );
    // LPDIRECTDRAWSURFACE FAR *lplpGDIDDSSurface

// --------------------------------------------------------------------------
// Method:          GetMonitorFrequency
// Description:     Call this function to retrieve the current monitor
//                  frequency
// Parameters:      monitorfrequency    - a pointer to a local variable that
//                                      is to be filled with the current
//                                      monitor frequency
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	GetMonitorFrequency( dword*
                                                        pMonitorFrequency );
    // LPDWORD lpdwFrequency

// --------------------------------------------------------------------------
// Method:          GetScanLine
// Description:     Call this function to retrieve the current monitor scan
//                  line
// Parameters:      scanline    - a pointer to a local variable that is to be
//                             filled with the current monitor scan line
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	GetScanLine( dword* pScanLine );
    // LPDWORD lpdwScanLine

// --------------------------------------------------------------------------
// Method:          GetVerticalBlankStatus
// Description:     Call this function to retrieve the current vertical blank
//                  status
// Parameters:      scanline    - a pointer to a local boolean variable that
//                              is to be filled with the current vertical
//                              blank status
// Notes:           if the result of this call is true, the monitor is
//                  currently inside the vertical blank period
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	GetVerticalBlankStatus( bool*
                                                    pVerticalBlankStatus );
    // LPBOOL lpbIsInVB

// --------------------------------------------------------------------------
// Method:          MessageHandler
// Description:     This is the TDirectDraw component's default message
//                  handling method
// Note:            The default implementation of this function is that it
//                  completely ignores the message
//                  Use the OnMessage event to override default message
//                  handling
// --------------------------------------------------------------------------
    void            __fastcall  MessageHandler( TObject *Sender,
                                                    AnsiString TheFunction,
                                                    AnsiString TheMessage );

// --------------------------------------------------------------------------
// Method:          Release
// Description:     Call this method to release the internals of the
//                  TDirectDraw object
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  Release();

// --------------------------------------------------------------------------
// Method:          RestoreDisplayMode
// Description:     This method will restore the display mode to the mode
//                  that was set before the application first changed the
//                  display mode
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	RestoreDisplayMode();

// --------------------------------------------------------------------------
// Method:          SetCooperativeLevel
// Description:     Determines the top-level behavior of the application
// Parameters:      Flags   - CooperativeLevel settings
//                  Handle  - applications handle, can be NULL if mode normal
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	SetCooperativeLevel( HWND pHandle,
                                                            dword pFlags );
    // HWND hWnd, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          SetDisplayMode
// Description:     This method can be used to change the current display
//                  mode settings
// Parameters:      width           - the width of the display mode to be set
//                  height          - the height of the display mode to be
//                                  set
//                  bitsperpixel    - the bits used per pixel of the display
//                                  mode to be set
//                  refreshrate     - the display mode refresh rate that will
//                                  be used
//                  flags           - can be DDSDM_STANDARDVGAMODE if
//                                  320x200x8 modeX is required
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  SetDisplayMode( dword pWidth, dword pHeight,
                                                        dword pBitsPerPixel,
                                                        dword pRefreshRate,
                                                        dword pFlags = 0 );
    // DWORD dwWidth, DWORD dwHeight, DWORD dwBPP, DWORD dwRefreshRate,
    //                                                          DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          WaitForVerticalBlank
// Description:     Use this function to help synchronize the application
//                  with the vertical-blank interval
// Parameters:      Flags   - which portion of the vertical blank to wait for
// --------------------------------------------------------------------------
    virtual	HRESULT	__fastcall	WaitForVerticalBlank( dword pFlags );
    // DWORD dwFlags, HANDLE hEvent

// --------------------------------------------------------------------------
// End of the TDirectDraw interface
// --------------------------------------------------------------------------

protected:

TDirectDrawSurface* __fastcall  FGetBackBuffer();
    void            __fastcall  FSetBackBuffer( TDirectDrawSurface*
                                                            pBackBuffer );
    dword           __fastcall  FGetBitsPerPixel();
    void            __fastcall  FSetBitsPerPixel( dword pBitsPerPixel );
    dword           __fastcall  FGetCooperativeLevel();
    void            __fastcall  FSetCooperativeLevel( dword
                                                        pCooperativeLevel );
    TDDSurfaceDesc* __fastcall  FGetCurrentDisplayModeInfo();
    void            __fastcall  FSetCurrentDisplayModeInfo( TDDSurfaceDesc*
                                                pCurrentDisplayModeInfo );      //added set
    TDDDisplayMode  __fastcall  FGetDisplayMode();
    void            __fastcall  FSetDisplayMode( TDDDisplayMode
                                                            pDisplayMode );
    dword           __fastcall  FGetHeight();
    void            __fastcall  FSetHeight( dword pHeight );
    dword           __fastcall  FGetMonitorFrequency();
    void            __fastcall  FSetMonitorFrequency( dword
                                                        pMonitorFrequency );    //added set
TDirectDrawSurface* __fastcall  FGetPrimarySurface();
    void            __fastcall  FSetPrimarySurface( TDirectDrawSurface*
                                                        pPrimarySurface );
    LPDIRECTDRAW2   __fastcall  FGetPtr();
    void            __fastcall  FSetPtr( LPDIRECTDRAW2 pPtr );
    dword           __fastcall  FGetRefreshRate();
    void            __fastcall  FSetRefreshRate( dword pRefreshRate );
    dword           __fastcall  FGetScanLine();
    void            __fastcall  FSetScanLine( dword pScanLine );                //added set
    bool            __fastcall  FGetUseFormHandle();
    void            __fastcall  FSetUseFormHandle( bool pUseFormHandle );
    bool            __fastcall  FGetVerticalBlankStatus();
    void            __fastcall  FSetVerticalBlankStatus( bool
                                                    pVerticalBlankStatus );     //added set
    dword           __fastcall  FGetWidth();
    void            __fastcall  FSetWidth( dword pWidth );

private:

    TDirectDrawEvent                FOnCreate;
    TDirectDrawEvent                FOnRelease;
    TDirectDrawMessage              FOnMessage;
    TDirectDrawError	            FOnError;
    TDirectDrawEnumCallback         FOnEnumCallback;
    TDirectDrawEnumCallbackEx       FOnEnumCallbackEx;
    TDirectDrawEnumModesCallback    FOnEnumModesCallback;
    TDirectDrawEnumSurfacesCallback FOnEnumSurfacesCallback;
    TDirectDrawEnumVideoCallback    FOnEnumVideoCallback;
    HWND                            fDefaultHandle;
    bool                            fUseFormHandle;
    bool                            fCreated;
	HRESULT				            fError;
	GUID				            fGUID;
	LPDIRECTDRAW		            fDirectDraw;
    LPDIRECTDRAW2                   fDirectDraw2;
    TDDCaps*                        fDriverCaps;
    TDDCaps*                        fHelCaps;
    TDirectDrawSurface*             fPrimarySurface;
    TDirectDrawSurface*             fBackBuffer;
    TDDDisplayMode                  fDisplayMode;
    dword                           fCooperativeLevel;
    dword                           fWidth;
    dword                           fHeight;
    dword                           fBitsPerPixel;
    dword                           fRefreshRate;
    
};
// --------------------------------------------------------------------------
#endif

